#!/bin/bash

CURRENT_DIR=$(cd $(dirname $0); pwd)

_unbind() {
	mdev_path="/sys/bus/mdev/devices/"
	mdevs=$(ls $mdev_path)
	for mdev in ${mdevs} ; do
		echo 1 > /sys/bus/mdev/devices/${mdev}/remove
	done

	devs=(`${CURRENT_DIR}/hyccp_devbind.py -s | sed -n '/CCP/p' | sed -n '/hymccp/p' | awk '{print $1}'`)
	for dev in ${devs[*]} ; do
		${CURRENT_DIR}/hyccp_devbind.py --unbind ${dev}
	done
}

_bind() {
	#bind ccp with hymccp driver
	if [ $2 -eq "0" ]; then
		devs=(`${CURRENT_DIR}/hyccp_devbind.py -s | sed -n '/CCP/p' | sed -n '/hymccp/!p' | grep "NTB" | awk '{print $1}'`)
		psp_devs=(`${CURRENT_DIR}/hyccp_devbind.py -s | sed -n '/CCP/p' | grep "drv=hymccp " | grep "PSP" | awk '{print $1}'`)
		for dev in ${psp_devs[*]} ; do
			${CURRENT_DIR}/hyccp_devbind.py --unbind ${dev}
		done
	else
		devs=(`${CURRENT_DIR}/hyccp_devbind.py -s | sed -n '/CCP/p' | sed -n '/hymccp/!p' | awk '{print $1}'`)
	fi
	for dev in ${devs[*]} ; do
		# fTPM issue
		if [ ${dev} == "${devs[0]}" ]; then
			continue;
		fi
		${CURRENT_DIR}/hyccp_devbind.py --bind=hymccp ${dev}
	done

	#create mdev per progress
	dev_num=`${CURRENT_DIR}/hyccp_devbind.py -s | sed -n '/hymccp/p' | awk 'END {print NR}'`
	echo "${dev_num} CCP devices are bound with hymccp driver"

	prog_num=$1
	mdev_num=$((${dev_num}*${prog_num}))
	for i in $(seq 1 ${mdev_num}) ; do
		echo `uuidgen` > /sys/devices/virtual/hymccp/hymccp/mdev_supported_types/hymccp-1/create
	done
	echo "${mdev_num} mdev devices are created to support max ${prog_num} progress"
}

_help() {
	echo "
usage:
  hymccp_bind.sh [-h]
               [-u]
               [-b NUM]
  -h, --help        show this help message and exit
  -b, --bind [num]  bind ccp with hymccp driver and create mdev
                    num specify max progresses using hymccp
  -p, --psp [opt]   enable or disable PSP device
                    opt could be "enable" or "disable". Default option is enable.
  -u, --unbind      unbind ccp from hymccp driver

example:
  bind ccp and create mdev to support max 32 progress
      hymccp_bind.sh --bind 32
  unbind ccp
      hymccp_bind.sh --unbind
  bind ccp only use ntb ccp
      hymccp_bind.sh --bind 32 --psp disable
  help
      hymccp_bind.sh --help
"
}


_insmod() {
	modprobe vfio
	modprobe vfio-pci
	modprobe vfio_iommu_type1
	modprobe mdev
	modprobe vfio_mdev

	if [ -d ${CURRENT_DIR}/../hymccp ]; then
		cd ${CURRENT_DIR}/../hymccp
		if [ -f ./Makefile ]; then
			make
		else
			echo "not found Makefile, skip compile"
		fi
		cp ./hymccp.ko ../scripts/
		if [ -f ./Makefile ]; then
			make clean
		fi
		cd ../scripts
		if [ -d "/sys/module/hymccp" ];then
			sudo rmmod hymccp
		fi
		sudo insmod ./hymccp.ko
		rm ./hymccp.ko
	else
		echo "not found ${CURRENT_DIR}/../hymccp, skip compile hymccp.ko"
		depmod
		modprobe hymccp
		mccp_inst=`lsmod | grep hymccp`
		if [ "$mccp_inst" ]; then
			return
		else
			echo "not found hymccp module installed in kernel, exit"
			exit
		fi
	fi

}

_process() {
	ARGS=`getopt -o hb:p:u --long help,bind:,psp:,unbind -- "$@"`
	if [ $? != 0 ]; then
		_help ;
		exit 1
	fi

	eval set -- "${ARGS}"

	let bindccp=0
	let mdevnum=0
	let psp=1

	while true; do
	case "$1" in
	-b|--bind)
		if [ "$2" -gt 0 ] 2>/dev/null ; then
			let bindccp=1
			let mdevnum=$2
		else
			_help ;
			exit 1
		fi
		shift 2
		;;
	-p|--psp)
		if [ "$2" = "enable" ]; then
			let psp=1
		elif [ "$2" = "disable" ]; then
			let psp=0
		fi
		shift 2
		;;
	-u|--unbind)
		_unbind ;
		shift
		;;
	-h|--help)
		_help ;
		exit 0
		shift
		;;
	--)
		shift
		break
		;;
	*)
		_help ;
		exit 1
		;;
	esac
	done

	if [ $bindccp -eq "1" ]; then
		_unbind && _insmod && _bind $mdevnum $psp
	fi
}

main() {
	if [ $# -lt 1 ]; then
		_help ;
		exit 1
	fi

	_process "$@"
}


main $*
